@@ -1,6 +1,5 @@
 /*
  *  Copyright (C) 2001,2002,2003 Nikos Mavroyanopoulos
- *  Copyright (C) 2004 Free Software Foundation
  *
  *  This file is part of GNUTLS.
  *
@@ -110,8 +109,10 @@ int _gnutls_copy_certificate_auth_info(CERTIFICATE_AUTH_INFO info,
  * -1 otherwise.
  */
 inline
-static int _gnutls_check_pk_algo_in_list(const gnutls_pk_algorithm *pk_algos, 
-	int pk_algos_length, gnutls_pk_algorithm algo_to_check)
+    static int _gnutls_check_pk_algo_in_list(gnutls_pk_algorithm *
+					     pk_algos, int pk_algos_length,
+					     gnutls_pk_algorithm
+					     algo_to_check)
 {
 	int i;
 	for (i = 0; i < pk_algos_length; i++) {
@@ -122,48 +123,6 @@ static int _gnutls_check_pk_algo_in_list(const gnutls_pk_algorithm *pk_algos,
 	return -1;
 }
 
-/* Returns the issuer's Distinguished name in odn, of the certificate 
- * specified in cert.
- */
-static int _gnutls_cert_get_issuer_dn(gnutls_cert * cert, gnutls_datum * odn )
-{
-	ASN1_TYPE dn;
-	int len, result;
-	int start, end;
-
-	if ((result=asn1_create_element
-	    (_gnutls_get_pkix(), "PKIX1.Certificate", &dn)) != ASN1_SUCCESS) {
-		gnutls_assert();
-		return _gnutls_asn2err(result);
-	}
-
-	result = asn1_der_decoding(&dn, cert->raw.data, cert->raw.size, NULL);
-	if (result != ASN1_SUCCESS) {
-		/* couldn't decode DER */
-		gnutls_assert();
-		asn1_delete_structure(&dn);
-		return _gnutls_asn2err(result);
-	}
-
-	result = asn1_der_decoding_startEnd(dn, cert->raw.data, cert->raw.size,
-					"tbsCertificate.issuer", &start,
-					&end);
-
-	if (result != ASN1_SUCCESS) {
-		/* couldn't decode DER */
-		gnutls_assert();
-		asn1_delete_structure(&dn);
-		return _gnutls_asn2err(result);
-	}
-	asn1_delete_structure(&dn);
-
-	len = end - start + 1;
-
-	odn->size = len;
-	odn->data = &cert->raw.data[start];
-
-	return 0;
-}
 
 
 /* Locates the most appropriate x509 certificate using the
@@ -171,7 +130,7 @@ static int _gnutls_cert_get_issuer_dn(gnutls_cert * cert, gnutls_datum * odn )
  */
 static int _find_x509_cert(const gnutls_certificate_credentials cred,
 			   opaque * _data, size_t _data_size,
-			   const gnutls_pk_algorithm * pk_algos,
+			   gnutls_pk_algorithm * pk_algos,
 			   int pk_algos_length, int *indx)
 {
 	uint size;
@@ -193,7 +152,7 @@ static int _find_x509_cert(const gnutls_certificate_credentials cred,
 		for (i = 0; i < cred->ncerts; i++) {
 			for (j = 0; j < cred->cert_list_length[i]; j++) {
 				if ((result =
-				     _gnutls_cert_get_issuer_dn(&cred->
+				     _gnutls_cert_get_dn(&cred->
 							 cert_list[i][j],
 							 &odn)) < 0) {
 					gnutls_assert();
@@ -284,7 +243,7 @@ uint size;
 	if (gnutls_certificate_type_get(session) != GNUTLS_CRT_X509)
 		return 0;
 
-	if (data_size > 0) do {
+	do {
 		/* This works like DECR_LEN() 
 		 */
 		result = GNUTLS_E_UNEXPECTED_PACKET_LENGTH;
@@ -441,17 +400,15 @@ static int call_client_cert_callback(gnutls_session session,
 
 OPENPGP_KEY_DEINIT _E_gnutls_openpgp_key_deinit;
 OPENPGP_PRIVKEY_DEINIT _E_gnutls_openpgp_privkey_deinit;
-
 /* Calls the client get callback.
  */
 static int call_get_cert_callback( gnutls_session session,
-	gnutls_datum* issuers_dn, int issuers_dn_length, 
-	gnutls_pk_algorithm* pk_algos, int pk_algos_length)
+	gnutls_datum* issuers_dn, int issuers_dn_length)
 {
 uint i;
 gnutls_cert *local_certs = NULL;
 gnutls_privkey *local_key = NULL;
-gnutls_retr_st st;
+retr_st st;
 int ret;
 gnutls_certificate_type type =
     gnutls_certificate_type_get(session);
@@ -473,7 +430,6 @@ const gnutls_certificate_credentials cred;
 		ret =
 		    cred->client_get_cert_callback(session,
 		    		issuers_dn, issuers_dn_length,
-				pk_algos, pk_algos_length,
 				&st);
 	}
 
@@ -482,9 +438,6 @@ const gnutls_certificate_credentials cred;
 		return GNUTLS_E_INTERNAL_ERROR;
 	}
 	
-	if (st.ncerts == 0)
-		return 0; /* no certificate was selected */
-	
 	if (type != st.type) {
 		gnutls_assert();
 		ret = GNUTLS_E_INVALID_REQUEST;
@@ -593,8 +546,7 @@ static int _select_client_cert(gnutls_session session,
 		}
 
 		if (cred->client_get_cert_callback) {
-			result = call_get_cert_callback( session, issuers_dn, issuers_dn_length,
-				pk_algos, pk_algos_length);
+			result = call_get_cert_callback( session, issuers_dn, issuers_dn_length);
 			goto cleanup;
 		}
 
@@ -1463,8 +1415,6 @@ static gnutls_cert *alloc_and_load_x509_certs(gnutls_x509_crt * certs,
 	gnutls_cert *local_certs;
 	int ret = 0;
 	uint i, j;
-	
-	if (certs==NULL) return NULL;
 
 	local_certs = gnutls_malloc(sizeof(gnutls_cert) * ncerts);
 	if (local_certs == NULL) {
@@ -1499,8 +1449,6 @@ static gnutls_privkey *alloc_and_load_x509_key(gnutls_x509_privkey key)
 	gnutls_privkey *local_key;
 	int ret = 0;
 
-	if (key==NULL) return NULL;
-
 	local_key = gnutls_malloc(sizeof(gnutls_privkey));
 	if (local_key == NULL) {
 		gnutls_assert();
@@ -1528,8 +1476,7 @@ static gnutls_cert *alloc_and_load_pgp_certs(gnutls_openpgp_key cert)
 {
 	gnutls_cert *local_certs;
 	int ret = 0;
-
-	if (cert==NULL) return NULL;
+	uint i, j;
 
 	local_certs = gnutls_malloc(sizeof(gnutls_cert));
 	if (local_certs == NULL) {
@@ -1542,7 +1489,7 @@ static gnutls_cert *alloc_and_load_pgp_certs(gnutls_openpgp_key cert)
 		return NULL;
 	}
 
-	ret = _E_gnutls_openpgp_key_to_gcert(local_certs, cert);
+	ret = _E_gnutls_openpgp_key_to_gcert(&local_certs[i], cert);
 	if (ret < 0) {
 		gnutls_assert();
 		return NULL;
@@ -1550,7 +1497,9 @@ static gnutls_cert *alloc_and_load_pgp_certs(gnutls_openpgp_key cert)
 
 	if (ret < 0) {
 		gnutls_assert();
-		_gnutls_gcert_deinit(local_certs);
+		for (j = 0; j < i; j++) {
+			_gnutls_gcert_deinit(&local_certs[j]);
+		}
 		gnutls_free(local_certs);
 		return NULL;
 	}
@@ -1566,8 +1515,6 @@ static gnutls_privkey *alloc_and_load_pgp_key(const gnutls_openpgp_privkey key)
 	gnutls_privkey *local_key;
 	int ret = 0;
 
-	if (key==NULL) return NULL;
-
 	local_key = gnutls_malloc(sizeof(gnutls_privkey));
 	if (local_key == NULL) {
 		gnutls_assert();
@@ -1673,7 +1620,7 @@ int _gnutls_server_select_cert(gnutls_session session,
 	 */
 	if (cred->server_get_cert_callback != NULL) {
 
-		return call_get_cert_callback( session, NULL, 0, NULL, 0);
+		return call_get_cert_callback( session, NULL, 0);
 
 	} else if (session->internals.server_cert_callback != NULL
 		   && cred->ncerts > 0) {
@@ -1701,7 +1648,7 @@ int _gnutls_server_select_cert(gnutls_session session,
 		j = 0;
 		for (i = 0; i < cred->ncerts; i++) {
 			/* Add compatible certificates */
-			if (requested_algo == GNUTLS_PK_ANY ||
+			if (requested_algo == (gnutls_pk_algorithm) - 1 ||
 			    requested_algo ==
 			    cred->cert_list[i][0].subject_pk_algorithm) {
 
