@@ -79,6 +79,7 @@ const MOD_AUTH_STRUCT dhe_dss_auth_struct = {
 static int gen_dhe_server_kx(gnutls_session session, opaque ** data)
 {
 	GNUTLS_MPI g, p;
+	const GNUTLS_MPI *mpis;
 	int ret = 0, data_size;
 	int bits;
 	gnutls_cert *apr_cert_list;
@@ -91,25 +92,29 @@ static int gen_dhe_server_kx(gnutls_session session, opaque ** data)
 	cred = _gnutls_get_cred(session->key, GNUTLS_CRD_CERTIFICATE, NULL);
 	if (cred == NULL) {
 	        gnutls_assert();
-	        return GNUTLS_E_INSUFICIENT_CREDENTIALS;
+	        return GNUTLS_E_INSUFFICIENT_CREDENTIALS;
 	}
 
 	bits = _gnutls_dh_get_prime_bits( session);
 
 	/* find the appropriate certificate */
 	if ((ret =
-	     _gnutls_find_apr_cert(session, &apr_cert_list,
+	     _gnutls_get_selected_cert(session, &apr_cert_list,
 				   &apr_cert_list_length,
 				   &apr_pkey)) < 0) {
 		gnutls_assert();
 		return ret;
 	}
 
-	if ( (ret=_gnutls_get_dh_params( cred->dh_params, &p, &g)) < 0) {
+	mpis = _gnutls_get_dh_params( cred->dh_params);
+	if (mpis == NULL) {
 		gnutls_assert();
-		return ret;
+		return GNUTLS_E_NO_TEMPORARY_DH_PARAMS;
 	}
 
+	p = mpis[0];
+	g = mpis[1];
+
 	if ( (ret=_gnutls_auth_info_set( session, GNUTLS_CRD_CERTIFICATE, 
 		sizeof( CERTIFICATE_AUTH_INFO_INT), 0)) < 0) 
 	{
@@ -125,8 +130,6 @@ static int gen_dhe_server_kx(gnutls_session session, opaque ** data)
 	}
 
 	ret = _gnutls_dh_common_print_server_kx( session, g, p, data);
-	_gnutls_mpi_release(&g);
-	_gnutls_mpi_release(&p);
 
 	if (ret < 0) {
 		gnutls_assert();
@@ -168,9 +171,6 @@ static int gen_dhe_server_kx(gnutls_session session, opaque ** data)
 	return data_size;
 }
 
-
-OPENPGP_CERT2GNUTLS_CERT _E_gnutls_openpgp_cert2gnutls_cert = NULL;
-
 static int proc_dhe_server_kx(gnutls_session session, opaque * data,
 				  size_t _data_size)
 {
@@ -205,32 +205,11 @@ static int proc_dhe_server_kx(gnutls_session session, opaque * data,
 	signature.data = &data[vparams.size + 2];
 	signature.size = sigsize;
 
-	switch( session->security_parameters.cert_type) {
-		case GNUTLS_CRT_X509:
-			if ((ret =
-			     _gnutls_x509_cert2gnutls_cert( &peer_cert,
-					     &info->raw_certificate_list[0], CERT_NO_COPY)) < 0) {
-				gnutls_assert();
-				return ret;
-			}
-			break;
-
-		case GNUTLS_CRT_OPENPGP:
-			if (_E_gnutls_openpgp_cert2gnutls_cert==NULL) {
-				gnutls_assert();
-				return GNUTLS_E_INIT_LIBEXTRA;
-			}
-			if ((ret =
-			     _E_gnutls_openpgp_cert2gnutls_cert( &peer_cert,
-					     &info->raw_certificate_list[0])) < 0) {
-				gnutls_assert();
-				return ret;
-			}
-			break;
-
-		default:
-			gnutls_assert();
-			return GNUTLS_E_INTERNAL_ERROR;
+	if ((ret =
+	     _gnutls_raw_cert_to_gcert( &peer_cert, session->security_parameters.cert_type,
+				     &info->raw_certificate_list[0], CERT_NO_COPY)) < 0) {
+		gnutls_assert();
+		return ret;
 	}
 
 	ret =
@@ -238,7 +217,7 @@ static int proc_dhe_server_kx(gnutls_session session, opaque * data,
 				      &peer_cert,
 				      &vparams, &signature);
 	
-	_gnutls_free_cert( &peer_cert);
+	_gnutls_gcert_deinit( &peer_cert);
 	if (ret < 0) {
 		gnutls_assert();
 		return ret;
@@ -256,23 +235,26 @@ int bits;
 const gnutls_certificate_credentials cred;
 int ret;
 GNUTLS_MPI p, g;
+const GNUTLS_MPI *mpis;
 
 	bits = _gnutls_dh_get_prime_bits( session);
 
 	cred = _gnutls_get_cred(session->key, GNUTLS_CRD_CERTIFICATE, NULL);
 	if (cred == NULL) {
 	        gnutls_assert();
-	        return GNUTLS_E_INSUFICIENT_CREDENTIALS;
+	        return GNUTLS_E_INSUFFICIENT_CREDENTIALS;
 	}
 
-	if ( (ret=_gnutls_get_dh_params( cred->dh_params, &p, &g)) < 0) {
+	mpis = _gnutls_get_dh_params( cred->dh_params);
+	if (mpis == NULL) {
 		gnutls_assert();
-		return ret;
+		return GNUTLS_E_NO_TEMPORARY_DH_PARAMS;
 	}
 
+	p = mpis[0];
+	g = mpis[1];
+
 	ret = _gnutls_proc_dh_common_client_kx( session, data, _data_size, g, p);
-	_gnutls_mpi_release(&g);
-	_gnutls_mpi_release(&p);
 	
 	return ret;
 
