@@ -1,6 +1,5 @@
 /*
  *  Copyright (C) 2000,2001,2003 Nikos Mavroyanopoulos
- *  Copyright (C) 2004 Free Software Foundation
  *
  *  This file is part of GNUTLS.
  *
@@ -51,6 +50,20 @@ int _gnutls_dh_generate_prime(GNUTLS_MPI * ret_g, GNUTLS_MPI * ret_n,
 	int result, times = 0, qbits;
 	GNUTLS_MPI *factors = NULL;
 	
+	g = mpi_new(16);	/* this should be ok */
+	if (g == NULL) {
+		gnutls_assert();
+		result = GNUTLS_E_MEMORY_ERROR;
+		goto cleanup;
+	}
+
+	prime = mpi_new(32);
+	if (prime == NULL) {
+		gnutls_assert();
+		result = GNUTLS_E_MEMORY_ERROR;
+		goto cleanup;
+	}	
+
 	/* Calculate the size of a prime factor of (prime-1)/2.
 	 * This is a bad emulation of Michael Wiener's table
 	 */
@@ -65,12 +78,6 @@ int _gnutls_dh_generate_prime(GNUTLS_MPI * ret_g, GNUTLS_MPI * ret_n,
 	/* find a prime number of size bits.
 	 */
 	do {
-	
-		if (times) {
-			_gnutls_mpi_release(&prime);
-			gcry_prime_release_factors (factors);
-		}
-
 		err = gcry_prime_generate( &prime, bits, qbits,
 			&factors, NULL, NULL, GCRY_STRONG_RANDOM,
 			GCRY_PRIME_FLAG_SPECIAL_FACTOR);
