@@ -1,6 +1,5 @@
 /*
  *  Copyright (C) 2000,2001,2002,2003 Nikos Mavroyanopoulos
- *  Copyright (C) 2004 Free Software Foundation
  *
  *  This file is part of GNUTLS.
  *
@@ -218,21 +217,18 @@ int _gnutls_finished(gnutls_session session, int type, void *ret)
 int _gnutls_create_random(opaque * dst)
 {
 	uint32 tim;
-
-	/* Use weak random numbers for the most of the
-	 * buffer except for the first 4 that are the
-	 * system's time.
-	 */
+	opaque rand[TLS_RANDOM_SIZE - 4];
 
 	tim = time(NULL);
 	/* generate server random value */
 	_gnutls_write_uint32(tim, dst);
 
 	if (_gnutls_get_random
-	    (&dst[4], TLS_RANDOM_SIZE - 4, GNUTLS_WEAK_RANDOM) < 0) {
+	    (rand, TLS_RANDOM_SIZE - 4, GNUTLS_STRONG_RANDOM) < 0) {
 		gnutls_assert();
 		return GNUTLS_E_MEMORY_ERROR;
 	}
+	memcpy(&dst[4], rand, TLS_RANDOM_SIZE - 4);
 
 	return 0;
 }
@@ -268,15 +264,9 @@ int _gnutls_read_client_hello(gnutls_session session, opaque * data,
 	/* if we do not support that version  */
 	if (_gnutls_version_is_supported(session, version) == 0) {
 		/* If he requested something we do not support
-		 * then we send him the highest we support.
+		 * then we send him the lowest we support.
 		 */
-		ver = _gnutls_version_max(session);
-		if (ver==GNUTLS_VERSION_UNKNOWN) {
-			/* this check is not really needed.
-			 */
-			gnutls_assert();
-			return GNUTLS_E_UNKNOWN_CIPHER_SUITE;
-		}
+		ver = _gnutls_version_lowest(session);
 	} else {
 		ver = version;
 	}
@@ -507,21 +497,14 @@ int _gnutls_recv_finished(gnutls_session session)
 }
 
 /* returns PK_RSA if the given cipher suite list only supports,
- * RSA algorithms, PK_DSA if DSS, and GNUTLS_PK_ANY if both or 
- * GNUTLS_PK_NONE for none.
+ * RSA algorithms, PK_DSA if DSS, and -1 if both or none.
  */
-static int _gnutls_server_find_pk_algos_in_ciphersuites( opaque* data, int datalen) 
-{
+int _gnutls_server_find_pk_algos_in_ciphersuites( opaque* data, int datalen) {
 int j;
-gnutls_pk_algorithm algo=GNUTLS_PK_NONE, prev_algo = 0;
+gnutls_pk_algorithm algo=-1, prev_algo = 0;
 gnutls_kx_algorithm kx;
 GNUTLS_CipherSuite cs;
 
-	if (datalen % 2 != 0) {
-		gnutls_assert();
-		return GNUTLS_E_UNEXPECTED_PACKET_LENGTH;
-	}
-
 	for (j = 0; j < datalen; j += 2) {
 		memcpy( &cs.CipherSuite, &data[j], 2);
 		kx = _gnutls_cipher_suite_get_kx_algo( cs);
@@ -529,7 +512,7 @@ GNUTLS_CipherSuite cs;
 		if ( _gnutls_map_kx_get_cred( kx, 1) == GNUTLS_CRD_CERTIFICATE) {
 			algo = _gnutls_map_pk_get_pk( kx);
 	
-			if (algo!=prev_algo && prev_algo!=0) return GNUTLS_PK_ANY;
+			if (algo!=prev_algo && prev_algo!=0) return -1;
 			prev_algo = algo;
 		}
 	}
@@ -570,11 +553,6 @@ int _gnutls_server_select_suite(gnutls_session session, opaque *data, int datale
 		else return GNUTLS_E_UNKNOWN_CIPHER_SUITE;
 	}
 
-	if (datalen % 2 != 0) {
-		gnutls_assert();
-		return GNUTLS_E_UNEXPECTED_PACKET_LENGTH;
-	}
-
 #ifdef HANDSHAKE_DEBUG
 
 	_gnutls_handshake_log("HSK[%x]: Requested cipher suites: \n", session);
@@ -1224,9 +1202,11 @@ opaque buf[2*TLS_MAX_SESSION_ID_SIZE+1];
 	_gnutls_handshake_log("HSK[%x]: SessionID: %s\n", session,
 		    _gnutls_bin2hex(session_id, session_id_len, buf, sizeof(buf)));
 
-	if (session_id_len > 0 &&
-        	session->internals.resumed_security_parameters.session_id_size == session_id_len &&
-		memcmp(session_id, session->internals.resumed_security_parameters.session_id,
+	if ((session->internals.resumed_security_parameters.
+	     session_id_size > 0)
+	    && memcmp(session_id,
+		      session->internals.
+		      resumed_security_parameters.session_id,
 		      session_id_len) == 0) {
 		/* resume session */
 		memcpy(session->internals.
@@ -1241,8 +1221,7 @@ opaque buf[2*TLS_MAX_SESSION_ID_SIZE+1];
 
 		return 0;
 	} else {
-		/* keep the new session id 
-		 */
+		/* keep the new session id */
 		session->internals.resumed = RESUME_FALSE;	/* we are not resuming */
 		session->security_parameters.session_id_size =
 		    session_id_len;
@@ -1302,6 +1281,7 @@ static int _gnutls_read_server_hello(gnutls_session session, opaque *data,
 	}
 	DECR_LEN(len, session_id_len);
 
+
 	/* check if we are resuming and set the appropriate
 	 * values;
 	 */
@@ -1352,7 +1332,7 @@ static int _gnutls_read_server_hello(gnutls_session session, opaque *data,
  * Needed in client hello messages. Returns the new data length.
  */
 static int _gnutls_copy_ciphersuites(gnutls_session session,
-				     opaque * ret_data, size_t ret_data_size)
+				     opaque ** ret_data)
 {
 	int ret, i;
 	GNUTLS_CipherSuite *cipher_suites;
@@ -1394,18 +1374,21 @@ static int _gnutls_copy_ciphersuites(gnutls_session session,
 
 	datalen += sizeof(uint16) + cipher_num;
 
-	if ((size_t)datalen > ret_data_size) {
+	*ret_data = gnutls_malloc(datalen);
+	if (*ret_data == NULL) {
 		gnutls_assert();
-		return GNUTLS_E_INTERNAL_ERROR;
+		gnutls_free(cipher_suites);
+		return GNUTLS_E_MEMORY_ERROR;
 	}
-	
-	_gnutls_write_uint16(cipher_num, ret_data);
+
+	_gnutls_write_uint16(cipher_num, *ret_data);
 	pos += 2;
 
 	for (i = 0; i < (cipher_num / 2); i++) {
-		memcpy( &ret_data[pos], cipher_suites[i].CipherSuite, 2);
+		memcpy( &(*ret_data)[pos], cipher_suites[i].CipherSuite, 2);
 		pos += 2;
 	}
+
 	gnutls_free(cipher_suites);
 
 	return datalen;
@@ -1416,7 +1399,7 @@ static int _gnutls_copy_ciphersuites(gnutls_session session,
  * Needed in hello messages. Returns the new data length.
  */
 static int _gnutls_copy_comp_methods(gnutls_session session,
-				     opaque * ret_data, size_t ret_data_size)
+				     opaque ** ret_data)
 {
 	int ret, i;
 	uint8 *compression_methods, comp_num;
@@ -1435,15 +1418,17 @@ static int _gnutls_copy_comp_methods(gnutls_session session,
 	datalen = pos = 0;
 	datalen += comp_num + 1;
 
-	if ((size_t)datalen > ret_data_size) {
+	*ret_data = gnutls_malloc(datalen);
+	if (*ret_data == NULL) {
 		gnutls_assert();
-		return GNUTLS_E_INTERNAL_ERROR;
+		gnutls_free(compression_methods);
+		return GNUTLS_E_MEMORY_ERROR;
 	}
 
-	ret_data[pos++] = comp_num; /* put the number of compression methods */
+	(*ret_data)[pos++] = comp_num; /* put the number of compression methods */
 
 	for (i = 0; i < comp_num; i++) {
-		ret_data[pos++] = compression_methods[i];
+		(*ret_data)[pos++] = compression_methods[i];
 	}
 
 	gnutls_free(compression_methods);
@@ -1451,22 +1436,17 @@ static int _gnutls_copy_comp_methods(gnutls_session session,
 	return datalen;
 }
 
-/* This should be sufficient by now. It should hold all the extensions
- * plus the headers in a hello message.
- */
-#define MAX_EXT_DATA_LENGTH 1024
-
 /* This function sends the client hello handshake message.
  */
 static int _gnutls_send_client_hello(gnutls_session session, int again)
 {
 	opaque *data = NULL;
+	opaque *extdata = NULL;
 	int extdatalen;
 	int pos = 0;
 	int datalen = 0, ret = 0;
 	opaque random[TLS_RANDOM_SIZE];
 	gnutls_protocol_version hver;
-	opaque extdata[MAX_EXT_DATA_LENGTH];
 
 	opaque *SessionID =
 	    session->internals.resumed_security_parameters.session_id;
@@ -1500,10 +1480,12 @@ static int _gnutls_send_client_hello(gnutls_session session, int again)
 			    resumed_security_parameters.version;
 		}
 
-		if (hver == GNUTLS_VERSION_UNKNOWN || hver == 0) {
+		if (hver <= 0) {
+			if (hver == 0)
+				hver = GNUTLS_E_INTERNAL_ERROR;
 			gnutls_assert();
 			gnutls_free(data);
-			return GNUTLS_E_INTERNAL_ERROR;
+			return hver;
 		}
 
 		data[pos++] = _gnutls_version_get_major(hver);
@@ -1548,16 +1530,18 @@ static int _gnutls_send_client_hello(gnutls_session session, int again)
 
 		/* Copy the ciphersuites.
 		 */
-		extdatalen = _gnutls_copy_ciphersuites(session, extdata, sizeof(extdata));
+		extdatalen = _gnutls_copy_ciphersuites(session, &extdata);
 		if (extdatalen > 0) {
 			datalen += extdatalen;
 			data = gnutls_realloc_fast(data, datalen);
 			if (data == NULL) {
 				gnutls_assert();
+				gnutls_free(extdata);
 				return GNUTLS_E_MEMORY_ERROR;
 			}
 
 			memcpy(&data[pos], extdata, extdatalen);
+			gnutls_free(extdata);
 			pos += extdatalen;
 
 		} else {
@@ -1571,16 +1555,18 @@ static int _gnutls_send_client_hello(gnutls_session session, int again)
 
 		/* Copy the compression methods.
 		 */
-		extdatalen = _gnutls_copy_comp_methods(session, extdata, sizeof(extdata));
+		extdatalen = _gnutls_copy_comp_methods(session, &extdata);
 		if (extdatalen > 0) {
 			datalen += extdatalen;
 			data = gnutls_realloc_fast(data, datalen);
 			if (data == NULL) {
 				gnutls_assert();
+				gnutls_free(extdata);
 				return GNUTLS_E_MEMORY_ERROR;
 			}
 
 			memcpy(&data[pos], extdata, extdatalen);
+			gnutls_free(extdata);
 			pos += extdatalen;
 
 		} else {
@@ -1594,17 +1580,19 @@ static int _gnutls_send_client_hello(gnutls_session session, int again)
 		/* Generate and copy TLS extensions.
 		 */
 		if (hver >= GNUTLS_TLS1) {
-			extdatalen = _gnutls_gen_extensions(session, extdata, sizeof(extdata));
+			extdatalen = _gnutls_gen_extensions(session, &extdata);
 
 			if (extdatalen > 0) {
 				datalen += extdatalen;
 				data = gnutls_realloc_fast(data, datalen);
 				if (data == NULL) {
 					gnutls_assert();
+					gnutls_free(extdata);
 					return GNUTLS_E_MEMORY_ERROR;
 				}
 
 				memcpy(&data[pos], extdata, extdatalen);
+				gnutls_free(extdata);
 			} else if (extdatalen < 0) {
 				gnutls_assert();
 				gnutls_free(data);
@@ -1624,7 +1612,7 @@ static int _gnutls_send_client_hello(gnutls_session session, int again)
 static int _gnutls_send_server_hello(gnutls_session session, int again)
 {
 	opaque *data = NULL;
-	opaque extdata[MAX_EXT_DATA_LENGTH];
+	opaque *extdata = NULL;
 	int extdatalen;
 	int pos = 0;
 	int datalen, ret = 0;
@@ -1638,14 +1626,11 @@ static int _gnutls_send_server_hello(gnutls_session session, int again)
 
 	datalen = 0;
 
-#ifdef ENABLE_SRP
 	if (IS_SRP_KX(
 		_gnutls_cipher_suite_get_kx_algo(
 			session->security_parameters.current_cipher_suite)))
 	{
-		if (session->internals.resumed == RESUME_FALSE &&
-			session->security_parameters.extensions.srp_username[0] == 0) 
-		{
+		if (session->security_parameters.extensions.srp_username[0] == 0) {
 			/* The peer didn't send a valid SRP extension with the
 			 * SRP username. The draft requires that we send an
 			 * alert and start the handshake again.
@@ -1661,11 +1646,10 @@ static int _gnutls_send_server_hello(gnutls_session session, int again)
 			return GNUTLS_E_INT_HANDSHAKE_AGAIN;
 		}
 	}
-#endif
 
 	if (again == 0) {
 		datalen = 2 + session_id_len + 1 + TLS_RANDOM_SIZE + 3;
-		extdatalen = _gnutls_gen_extensions(session, extdata, sizeof(extdata));
+		extdatalen = _gnutls_gen_extensions(session, &extdata);
 
 		if (extdatalen < 0) {
 			gnutls_assert();
@@ -1675,6 +1659,7 @@ static int _gnutls_send_server_hello(gnutls_session session, int again)
 		data = gnutls_alloca(datalen + extdatalen);
 		if (data == NULL) {
 			gnutls_assert();
+			gnutls_free(extdata);
 			return GNUTLS_E_MEMORY_ERROR;
 		}
 
@@ -1715,6 +1700,7 @@ static int _gnutls_send_server_hello(gnutls_session session, int again)
 			datalen += extdatalen;
 
 			memcpy(&data[pos], extdata, extdatalen);
+			gnutls_free(extdata);
 		}
 	}
 
@@ -1791,7 +1777,7 @@ int _gnutls_recv_hello(gnutls_session session, opaque *data, int datalen)
 
 /**
   * gnutls_rehandshake - This function will renegotiate security parameters
-  * @session: is a &gnutls_session structure.
+  * @session: is a a &gnutls_session structure.
   *
   * This function will renegotiate security parameters with the
   * client. This should only be called in case of a server.
@@ -1832,9 +1818,7 @@ int gnutls_rehandshake(gnutls_session session)
 	return 0;
 }
 
-inline 
-static int _gnutls_abort_handshake( gnutls_session session, int ret) 
-{
+static int _gnutls_abort_handshake( gnutls_session session, int ret) {
 	if ( ((ret==GNUTLS_E_WARNING_ALERT_RECEIVED) && 
 		( gnutls_alert_get(session) == GNUTLS_A_NO_RENEGOTIATION))
 		|| ret==GNUTLS_E_GOT_APPLICATION_DATA)
@@ -1911,11 +1895,10 @@ int gnutls_handshake(gnutls_session session)
 	}
 	if (ret < 0) {
 		/* In the case of a rehandshake abort
-		 * we should reset the handshake's internal state.
+		 * we should reset the handshake's session
 		 */
 		if (_gnutls_abort_handshake( session, ret) == 0)
 			STATE = STATE0;
-
 		return ret;
 	}
 	
@@ -1924,7 +1907,6 @@ int gnutls_handshake(gnutls_session session)
 	if (ret < 0) {
 		if (_gnutls_abort_handshake( session, ret) == 0)
 			STATE = STATE0;
-		
 		return ret;
 	}
 	
@@ -2367,15 +2349,13 @@ inline static int check_server_params( gnutls_session session, gnutls_kx_algorit
 
 	cred_type = _gnutls_map_kx_get_cred( kx, 1);
 	
-	/* Read the DH parameters if any.
-	 */
 	if (cred_type == GNUTLS_CRD_CERTIFICATE) {
 		x509_cred =
 		    _gnutls_get_cred(session->key, cred_type, NULL);
 	
 		if (x509_cred != NULL) {
-			dh_params = _gnutls_certificate_get_dh_params(x509_cred, session);
-			rsa_params = _gnutls_certificate_get_rsa_params(x509_cred, session);
+			dh_params = x509_cred->dh_params;
+			rsa_params = x509_cred->rsa_params;
 		}
 
 		/* Check also if the certificate supports the
@@ -2396,11 +2376,9 @@ inline static int check_server_params( gnutls_session session, gnutls_kx_algorit
 		anon_cred =
 		    _gnutls_get_cred(session->key, cred_type, NULL);
 	
-#ifdef ENABLE_ANON
 		if (anon_cred != NULL) {
-			dh_params = _gnutls_anon_get_dh_params(anon_cred, session);
+			dh_params = anon_cred->dh_params;
 		}
-#endif
 	} else return 0; /* no need for params */
 
 
@@ -2533,7 +2511,7 @@ int _gnutls_remove_unwanted_ciphersuites(gnutls_session session,
 
 /**
   * gnutls_handshake_set_max_packet_length - This function will set the maximum length of a handshake message
-  * @session: is a &gnutls_session structure.
+  * @session: is a a &gnutls_session structure.
   * @max: is the maximum number.
   *
   * This function will set the maximum size of a handshake message.
