@@ -92,11 +92,6 @@ typedef void * gnutls_transport_ptr;
 
 #define HANDSHAKE_HEADER_SIZE 4
 
-/* defaults for verification functions
- */
-#define DEFAULT_VERIFY_DEPTH 6
-#define DEFAULT_VERIFY_BITS 8200
-
 #include <gnutls_mem.h>
 #include <gnutls_ui.h>
 
@@ -107,11 +102,6 @@ typedef void * gnutls_transport_ptr;
 typedef unsigned char opaque;
 typedef struct { opaque pint[3]; } uint24;
 
-typedef struct {
-	opaque * data;
-	unsigned int size;
-} gnutls_datum;
-
 #include <gnutls_mpi.h>
 
 typedef enum ChangeCipherSpecType { GNUTLS_TYPE_CHANGE_CIPHER_SPEC=1 } ChangeCipherSpecType;
@@ -151,6 +141,16 @@ typedef enum HandshakeType { GNUTLS_HELLO_REQUEST, GNUTLS_CLIENT_HELLO, GNUTLS_S
 
 typedef HandshakeType gnutls_handshake_description;
 
+typedef struct {
+	opaque * data;
+	unsigned int size;
+} gnutls_datum;
+
+typedef struct {
+	const opaque * data;
+	unsigned int size;
+} gnutls_const_datum;
+
 #include <gnutls_buffer.h>
 
 /* This is the maximum number of algorithms (ciphers or macs etc).
@@ -174,16 +174,12 @@ typedef enum gnutls_kx_algorithm { GNUTLS_KX_RSA=1, GNUTLS_KX_DHE_DSS,
 	GNUTLS_KX_RSA_EXPORT, GNUTLS_KX_SRP_RSA, GNUTLS_KX_SRP_DSS
 } gnutls_kx_algorithm;
 
-typedef enum gnutls_params_type { GNUTLS_PARAMS_RSA_EXPORT=1,
-	GNUTLS_PARAMS_DH 
-} gnutls_params_type;
-
 typedef enum gnutls_mac_algorithm { GNUTLS_MAC_UNKNOWN=0, GNUTLS_MAC_NULL=1, 
 GNUTLS_MAC_MD5, GNUTLS_MAC_SHA, GNUTLS_MAC_RMD160
 } gnutls_mac_algorithm;
 typedef gnutls_mac_algorithm gnutls_digest_algorithm;
 
-typedef enum gnutls_compression_method { GNUTLS_COMP_NULL=1, GNUTLS_COMP_DEFLATE,
+typedef enum gnutls_compression_method { GNUTLS_COMP_NULL=1, GNUTLS_COMP_ZLIB,
 	GNUTLS_COMP_LZO
 } gnutls_compression_method;
 
@@ -215,13 +211,6 @@ typedef enum gnutls_x509_crt_fmt { GNUTLS_X509_FMT_DER,
 typedef enum gnutls_pk_algorithm { GNUTLS_PK_RSA = 1, GNUTLS_PK_DSA,
 	GNUTLS_PK_UNKNOWN = 0xff
 } gnutls_pk_algorithm;
-#define GNUTLS_PK_ANY (gnutls_pk_algorithm)-1
-#define GNUTLS_PK_NONE (gnutls_pk_algorithm)0
-
-typedef enum gnutls_sign_algorithm { GNUTLS_SIGN_RSA_SHA = 1, GNUTLS_SIGN_DSA_SHA,
-	GNUTLS_SIGN_RSA_MD5, GNUTLS_SIGN_RSA_MD2,
-	GNUTLS_SIGN_UNKNOWN = 0xff
-} gnutls_sign_algorithm;
 
 /* STATE (stop) */
 
@@ -309,9 +298,7 @@ typedef struct {
 /* Versions should be in order of the oldest
  * (eg. SSL3 is before TLS1)
  */
-#define GNUTLS_TLS1 GNUTLS_TLS1_0
-typedef enum gnutls_protocol_version { GNUTLS_SSL3=1, GNUTLS_TLS1_0, 
-	GNUTLS_TLS1_1, GNUTLS_VERSION_UNKNOWN=0xff } 
+typedef enum gnutls_protocol_version { GNUTLS_SSL3=1, GNUTLS_TLS1, GNUTLS_VERSION_UNKNOWN=0xff } 
 gnutls_protocol_version;
 
 /* This structure holds parameters got from TLS extension
@@ -433,38 +420,6 @@ typedef int certificate_server_select_func(struct gnutls_session_int*,
 typedef int srp_server_select_func(struct gnutls_session_int*, 
 	const char**, const char**, unsigned int);
 
-/* DH and RSA parameters types.
- */
-typedef struct {
-	/* [0] is the prime, [1] is the generator.
-	 */
-	GNUTLS_MPI params[2];
-} _gnutls_dh_params;
-
-#define gnutls_dh_params _gnutls_dh_params*
-
-#define gnutls_rsa_params gnutls_x509_privkey
-
-typedef struct gnutls_internal_params {
-	gnutls_dh_params	anon_dh_params;
-	int			free_anon_dh_params;
-	gnutls_dh_params	cert_dh_params;
-	int			free_cert_dh_params;
-	gnutls_rsa_params	rsa_params;
-	int 			free_rsa_params;
-} gnutls_internal_params;
-
-typedef struct gnutls_params_st {
-	gnutls_params_type type;
-	union params {
-		gnutls_dh_params dh;
-		gnutls_rsa_params rsa_export;
-	} params;
-	int deinit;
-} gnutls_params_st;
-
-
-
 typedef struct {
 	opaque				header[HANDSHAKE_HEADER_SIZE];
 	/* this holds the number of bytes in the handshake_header[] */
@@ -619,7 +574,7 @@ typedef struct {
 	 */
 	uint16			proposed_record_size;
 	
-	/* holds the selected certificate and key.
+	/* holds the the selected certificate and key.
 	 * use _gnutls_selected_certs_deinit() and _gnutls_selected_certs_set()
 	 * to change them.
 	 */
@@ -669,7 +624,7 @@ typedef struct {
 
 	/* This is used to set an arbitary version in the RSA
 	 * PMS secret. Can be used by clients to test whether the
-	 * server checks that version. (** only used in gnutls-cli-debug)
+	 * server checks that version.
 	 */
 	opaque			rsa_pms_version[2];
 
@@ -681,12 +636,6 @@ typedef struct {
 	 */
 	int			handshake_restarted;
 
-	/* Here we cache the DH or RSA parameters got from the
-	 * credentials structure, or from a callback. That is to
-	 * minimize external calls.
-	 */
-	gnutls_internal_params  params;
-
 	/* If you add anything here, check _gnutls_handshake_internal_state_clear().
 	 */
 } GNUTLS_INTERNALS;
@@ -701,10 +650,18 @@ struct gnutls_session_int {
 
 typedef struct gnutls_session_int *gnutls_session;
 
+typedef struct {
+	/* [0] is the prime, [1] is the generator.
+	 */
+	GNUTLS_MPI params[2];
+} _gnutls_dh_params;
+
+#define gnutls_dh_params _gnutls_dh_params*
+
+#define gnutls_rsa_params gnutls_x509_privkey
 
 
-/* functions 
- */
+/* functions */
 void _gnutls_set_current_version(gnutls_session session, gnutls_protocol_version version);
 gnutls_protocol_version gnutls_protocol_get_version(gnutls_session session);
 void _gnutls_free_auth_info( gnutls_session session);
@@ -728,7 +685,4 @@ gnutls_protocol_version _gnutls_get_adv_version( gnutls_session);
 int gnutls_fingerprint(gnutls_digest_algorithm algo, const gnutls_datum* data, 
 	void* result, size_t* result_size);
 
-typedef int gnutls_params_function(gnutls_session, gnutls_params_type,
-	gnutls_params_st*);
-
 #endif /* GNUTLS_INT_H */
