@@ -84,17 +84,6 @@ const ASN1_ARRAY_TYPE pkix_asn1_tab[]={
   {0,536870917,0},
   {"issuerDomainPolicy",1073741826,"CertPolicyId"},
   {"subjectDomainPolicy",2,"CertPolicyId"},
-  {"DirectoryString",1610612754,0},
-  {"teletexString",1612709890,"TeletexString"},
-  {"MAX",524298,"1"},
-  {"printableString",1612709890,"PrintableString"},
-  {"MAX",524298,"1"},
-  {"universalString",1612709890,"UniversalString"},
-  {"MAX",524298,"1"},
-  {"utf8String",1612709890,"UTF8String"},
-  {"MAX",524298,"1"},
-  {"bmpString",538968066,"BMPString"},
-  {"MAX",524298,"1"},
   {"id-ce-subjectAltName",1879048204,0},
   {0,1073741825,"id-ce"},
   {0,1,"17"},
@@ -192,10 +181,25 @@ const ASN1_ARRAY_TYPE pkix_asn1_tab[]={
   {0,4104,"1"},
   {"cRLIssuer",536895490,"GeneralNames"},
   {0,4104,"2"},
-  {"DistributionPointName",1610612741,0},
-  {"fullName",1610637314,"GeneralNames"},
+  {"DistributionPointName",1610612754,0},
+  {"fullName",1610620930,"GeneralNames"},
   {0,4104,"0"},
-  {"nameRelativeToCRLIssuer",536895490,"RelativeDistinguishedName"},
+  {"nameRelativeToCRLIssuer",536879106,"RelativeDistinguishedName"},
+  {0,4104,"1"},
+  {"WritableCRLDistributionPoints",1612709899,0},
+  {"MAX",1074266122,"1"},
+  {0,2,"WritableDistributionPoint"},
+  {"WritableDistributionPoint",1610612741,0},
+  {"distributionPoint",1610637314,"WriteableDistributionPointName"},
+  {0,4104,"0"},
+  {"reasons",1610637314,"ReasonFlags"},
+  {0,4104,"1"},
+  {"cRLIssuer",536895490,"GeneralNames"},
+  {0,4104,"2"},
+  {"WriteableDistributionPointName",1610612754,0},
+  {"fullName",1610620941,0},
+  {0,4104,"0"},
+  {"nameRelativeToCRLIssuer",536879106,"RelativeDistinguishedName"},
   {0,4104,"1"},
   {"ReasonFlags",1610874886,0},
   {"unused",1073741825,"0"},
@@ -374,50 +378,116 @@ const ASN1_ARRAY_TYPE pkix_asn1_tab[]={
   {"joint-iso-ccitt",1073741825,"2"},
   {"ds",1073741825,"5"},
   {0,1,"4"},
-  {"id-at-initials",1880096780,"AttributeType"},
-  {0,1073741825,"id-at"},
-  {0,1,"43"},
-  {"X520initials",1073741826,"DirectoryString"},
-  {"id-at-generationQualifier",1880096780,"AttributeType"},
-  {0,1073741825,"id-at"},
-  {0,1,"44"},
-  {"X520generationQualifier",1073741826,"DirectoryString"},
-  {"id-at-surname",1880096780,"AttributeType"},
-  {0,1073741825,"id-at"},
-  {0,1,"4"},
-  {"X520surName",1073741826,"DirectoryString"},
-  {"id-at-givenName",1880096780,"AttributeType"},
-  {0,1073741825,"id-at"},
-  {0,1,"42"},
-  {"X520givenName",1073741826,"DirectoryString"},
   {"id-at-name",1880096780,"AttributeType"},
   {0,1073741825,"id-at"},
   {0,1,"41"},
-  {"X520name",1073741826,"DirectoryString"},
+  {"id-at-surname",1880096780,"AttributeType"},
+  {0,1073741825,"id-at"},
+  {0,1,"4"},
+  {"id-at-givenName",1880096780,"AttributeType"},
+  {0,1073741825,"id-at"},
+  {0,1,"42"},
+  {"id-at-initials",1880096780,"AttributeType"},
+  {0,1073741825,"id-at"},
+  {0,1,"43"},
+  {"id-at-generationQualifier",1880096780,"AttributeType"},
+  {0,1073741825,"id-at"},
+  {0,1,"44"},
+  {"X520name",1610612754,0},
+  {"teletexString",1612709890,"TeletexString"},
+  {"ub-name",524298,"1"},
+  {"printableString",1612709890,"PrintableString"},
+  {"ub-name",524298,"1"},
+  {"universalString",1612709890,"UniversalString"},
+  {"ub-name",524298,"1"},
+  {"utf8String",1612709890,"UTF8String"},
+  {"ub-name",524298,"1"},
+  {"bmpString",538968066,"BMPString"},
+  {"ub-name",524298,"1"},
   {"id-at-commonName",1880096780,"AttributeType"},
   {0,1073741825,"id-at"},
   {0,1,"3"},
-  {"X520CommonName",1073741826,"DirectoryString"},
+  {"X520CommonName",1610612754,0},
+  {"teletexString",1612709890,"TeletexString"},
+  {"ub-common-name",524298,"1"},
+  {"printableString",1612709890,"PrintableString"},
+  {"ub-common-name",524298,"1"},
+  {"universalString",1612709890,"UniversalString"},
+  {"ub-common-name",524298,"1"},
+  {"utf8String",1612709890,"UTF8String"},
+  {"ub-common-name",524298,"1"},
+  {"bmpString",538968066,"BMPString"},
+  {"ub-common-name",524298,"1"},
   {"id-at-localityName",1880096780,"AttributeType"},
   {0,1073741825,"id-at"},
   {0,1,"7"},
-  {"X520LocalityName",1073741826,"DirectoryString"},
+  {"X520LocalityName",1610612754,0},
+  {"teletexString",1612709890,"TeletexString"},
+  {"ub-locality-name",524298,"1"},
+  {"printableString",1612709890,"PrintableString"},
+  {"ub-locality-name",524298,"1"},
+  {"universalString",1612709890,"UniversalString"},
+  {"ub-locality-name",524298,"1"},
+  {"utf8String",1612709890,"UTF8String"},
+  {"ub-locality-name",524298,"1"},
+  {"bmpString",538968066,"BMPString"},
+  {"ub-locality-name",524298,"1"},
   {"id-at-stateOrProvinceName",1880096780,"AttributeType"},
   {0,1073741825,"id-at"},
   {0,1,"8"},
-  {"X520StateOrProvinceName",1073741826,"DirectoryString"},
+  {"X520StateOrProvinceName",1610612754,0},
+  {"teletexString",1612709890,"TeletexString"},
+  {"ub-state-name",524298,"1"},
+  {"printableString",1612709890,"PrintableString"},
+  {"ub-state-name",524298,"1"},
+  {"universalString",1612709890,"UniversalString"},
+  {"ub-state-name",524298,"1"},
+  {"utf8String",1612709890,"UTF8String"},
+  {"ub-state-name",524298,"1"},
+  {"bmpString",538968066,"BMPString"},
+  {"ub-state-name",524298,"1"},
   {"id-at-organizationName",1880096780,"AttributeType"},
   {0,1073741825,"id-at"},
   {0,1,"10"},
-  {"X520OrganizationName",1073741826,"DirectoryString"},
+  {"X520OrganizationName",1610612754,0},
+  {"teletexString",1612709890,"TeletexString"},
+  {"ub-organization-name",524298,"1"},
+  {"printableString",1612709890,"PrintableString"},
+  {"ub-organization-name",524298,"1"},
+  {"universalString",1612709890,"UniversalString"},
+  {"ub-organization-name",524298,"1"},
+  {"utf8String",1612709890,"UTF8String"},
+  {"ub-organization-name",524298,"1"},
+  {"bmpString",538968066,"BMPString"},
+  {"ub-organization-name",524298,"1"},
   {"id-at-organizationalUnitName",1880096780,"AttributeType"},
   {0,1073741825,"id-at"},
   {0,1,"11"},
-  {"X520OrganizationalUnitName",1073741826,"DirectoryString"},
+  {"X520OrganizationalUnitName",1610612754,0},
+  {"teletexString",1612709890,"TeletexString"},
+  {"ub-organizational-unit-name",524298,"1"},
+  {"printableString",1612709890,"PrintableString"},
+  {"ub-organizational-unit-name",524298,"1"},
+  {"universalString",1612709890,"UniversalString"},
+  {"ub-organizational-unit-name",524298,"1"},
+  {"utf8String",1612709890,"UTF8String"},
+  {"ub-organizational-unit-name",524298,"1"},
+  {"bmpString",538968066,"BMPString"},
+  {"ub-organizational-unit-name",524298,"1"},
   {"id-at-title",1880096780,"AttributeType"},
   {0,1073741825,"id-at"},
   {0,1,"12"},
-  {"X520Title",1073741826,"DirectoryString"},
+  {"X520Title",1610612754,0},
+  {"teletexString",1612709890,"TeletexString"},
+  {"ub-title",524298,"1"},
+  {"printableString",1612709890,"PrintableString"},
+  {"ub-title",524298,"1"},
+  {"universalString",1612709890,"UniversalString"},
+  {"ub-title",524298,"1"},
+  {"utf8String",1612709890,"UTF8String"},
+  {"ub-title",524298,"1"},
+  {"bmpString",538968066,"BMPString"},
+  {"ub-title",524298,"1"},
   {"id-at-dnQualifier",1880096780,"AttributeType"},
   {0,1073741825,"id-at"},
   {0,1,"46"},
@@ -427,20 +497,6 @@ const ASN1_ARRAY_TYPE pkix_asn1_tab[]={
   {0,1,"6"},
   {"X520countryName",1612709890,"PrintableString"},
   {0,1048586,"2"},
-  {"id-at-pseudonym",1880096780,"AttributeType"},
-  {0,1073741825,"id-at"},
-  {0,1,"65"},
-  {"X520pseudonym",1073741826,"DirectoryString"},
-  {"id-at-streetAddress",1880096780,"AttributeType"},
-  {0,1073741825,"id-at"},
-  {0,1,"9"},
-  {"X520streetAddress",1073741826,"DirectoryString"},
-  {"id-at-postalAddress",1880096780,"AttributeType"},
-  {0,1073741825,"id-at"},
-  {0,1,"16"},
-  {"X520postalAddress",1073741826,"PostalAddress"},
-  {"PostalAddress",1610612747,0},
-  {0,2,"DirectoryString"},
   {"pkcs",1879048204,0},
   {"iso",1073741825,"1"},
   {"member-body",1073741825,"2"},
@@ -463,6 +519,17 @@ const ASN1_ARRAY_TYPE pkix_asn1_tab[]={
   {"RelativeDistinguishedName",1612709903,0},
   {"MAX",1074266122,"1"},
   {0,2,"AttributeTypeAndValue"},
+  {"DirectoryString",1610612754,0},
+  {"teletexString",1612709890,"TeletexString"},
+  {"MAX",524298,"1"},
+  {"printableString",1612709890,"PrintableString"},
+  {"MAX",524298,"1"},
+  {"universalString",1612709890,"UniversalString"},
+  {"MAX",524298,"1"},
+  {"utf8String",1612709890,"UTF8String"},
+  {"MAX",524298,"1"},
+  {"bmpString",538968066,"BMPString"},
+  {"MAX",524298,"1"},
   {"Certificate",1610612741,0},
   {"tbsCertificate",1073741826,"TBSCertificate"},
   {"signatureAlgorithm",1073741826,"AlgorithmIdentifier"},
@@ -1019,52 +1086,8 @@ const ASN1_ARRAY_TYPE pkix_asn1_tab[]={
   {0,4104,"0"},
   {"pkcs-7-ContentEncryptionAlgorithmIdentifier",1073741826,"AlgorithmIdentifier"},
   {"pkcs-7-EncryptedContent",1073741831,0},
-  {"pkcs-7-UnprotectedAttributes",1612709903,0},
+  {"pkcs-7-UnprotectedAttributes",538968079,0},
   {"MAX",1074266122,"1"},
   {0,2,"Attribute"},
-  {"id-at-ldap-DC",1880096780,"AttributeType"},
-  {0,1073741825,"0"},
-  {0,1073741825,"9"},
-  {0,1073741825,"2342"},
-  {0,1073741825,"19200300"},
-  {0,1073741825,"100"},
-  {0,1073741825,"1"},
-  {0,1,"25"},
-  {"ldap-DC",1073741826,"IA5String"},
-  {"id-at-ldap-UID",1880096780,"AttributeType"},
-  {0,1073741825,"0"},
-  {0,1073741825,"9"},
-  {0,1073741825,"2342"},
-  {0,1073741825,"19200300"},
-  {0,1073741825,"100"},
-  {0,1073741825,"1"},
-  {0,1,"1"},
-  {"ldap-UID",1073741826,"IA5String"},
-  {"id-pda",1879048204,0},
-  {0,1073741825,"id-pkix"},
-  {0,1,"9"},
-  {"id-pda-dateOfBirth",1880096780,"AttributeType"},
-  {0,1073741825,"id-pda"},
-  {0,1,"1"},
-  {"DateOfBirth",1082130449,0},
-  {"id-pda-placeOfBirth",1880096780,"AttributeType"},
-  {0,1073741825,"id-pda"},
-  {0,1,"2"},
-  {"PlaceOfBirth",1073741826,"DirectoryString"},
-  {"id-pda-gender",1880096780,"AttributeType"},
-  {0,1073741825,"id-pda"},
-  {0,1,"3"},
-  {"Gender",1612709890,"PrintableString"},
-  {0,1048586,"1"},
-  {"id-pda-countryOfCitizenship",1880096780,"AttributeType"},
-  {0,1073741825,"id-pda"},
-  {0,1,"4"},
-  {"CountryOfCitizenship",1612709890,"PrintableString"},
-  {0,1048586,"2"},
-  {"id-pda-countryOfResidence",1880096780,"AttributeType"},
-  {0,1073741825,"id-pda"},
-  {0,1,"5"},
-  {"CountryOfResidence",538968066,"PrintableString"},
-  {0,1048586,"2"},
   {0,0,0}
 };
